 (******************************************************
 *                                                     *
 *   Academy Component Library                         *
 *   Copyright (c) 1996, Academy Corporation Limited   *
 *                                                     *
 *       Class: TADBGrid           Data Bound: YES     *
 *    Ancestor: TDBGrid                                *
 *                                                     *
 *   Author: Matthew J O'Haire                         *
 *                                                     *
 ******************************************************)


unit adbgrid;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, DBGrids;

type
  { new event type for column header click with mouse state & column index }
  TColumnClickEvent = procedure(Sender: TObject; Index: LongInt;
     Button: TMouseButton; Shift: TShiftState) of object;

  { new DBGrid class }
  TADBGrid = class(TDBGrid)
    FColumnClick: integer;
    FButton: TMouseButton;
    FShift: TShiftState;
    FOnColumnClick: TColumnClickEvent;
    procedure DoMouseDown(var Message: TWMMouse; Button: TMouseButton;
      Shift: TShiftState);

  protected
    { override the standard DBGrid mouse event handling }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;

    { add new win message handlers for mouse clicks }
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMRButtonDown(var Message: TWMRButtonDown); message WM_RBUTTONDOWN;
    procedure WMMButtonDown(var Message: TWMMButtonDown); message WM_MBUTTONDOWN;

  public
    constructor Create(AOwner: TComponent); override;
    property Canvas;
    property SelectedRows;
    { MouseToCell method as per TStringGrid }
    procedure MouseToCell(X, Y: Integer; var ACol, ARow: Longint); dynamic;

  published
    // surface all current DBGrid properties & events
    property Align;
    property BorderStyle;
    property Color;
    property Columns stored StoreColumns;
    property Ctl3D;
    property DataSource;
    property DefaultDrawing;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FixedColor;
    property Font;
    property Options;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property TitleFont;
    property Visible;
    property OnColEnter;
    property OnColExit;
    property OnColumnMoved;
    property OnDrawDataCell;  { obsolete }
    property OnDrawColumnCell;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEditButtonClick;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
    // publish new events
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnColumnClick: TColumnClickEvent read FOnColumnClick write FOnColumnClick;
  end;

procedure Register;


implementation

{$R *.RES}

{ constructor/destructor methods }

constructor TADBGrid.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   // set initial state
   FColumnClick := -1;  // not monitoring a column click
end;


{ private methods }

procedure TADBGrid.DoMouseDown(var Message: TWMMouse; Button: TMouseButton;
  Shift: TShiftState);
begin
  if not (csNoStdEvents in ControlStyle) then
    with Message do begin
      MouseDown(Button, KeysToShiftState(Keys) + Shift, XPos, YPos);
      if assigned(OnMouseDown) then
         OnMouseDown(self, Button, KeysToShiftState(Keys) + Shift, XPos, YPos);
    end;
end;


{ protected methods }

procedure TADBGrid.MouseDown(Button: TMouseButton;
                              Shift: TShiftState;
                               X, Y: Integer);
var
   ACol, ARow: longint;
begin
   inherited;
   MouseToCell(X, Y, ACol, ARow);

   // if down on a column header (row 0) then start watching for a "click"
   if ARow = 0 then begin
      FColumnClick := ACol;
      FButton := Button;
      FShift := Shift;
   end;
end;

procedure TADBGrid.MouseMove(Shift: TShiftState; X, Y: Integer);
var
   ACol, ARow: longint;
begin
   inherited;

   // monitor column click movement
   if (FColumnClick >= 0) then begin
      MouseToCell(X, Y, ACol, ARow);
      if ((ACol <> FColumnClick) or (ARow <> 0)) then  // out of column/title
         FColumnClick := -1;  // finish "click"
   end;
end;

procedure TADBGrid.MouseUp(Button: TMouseButton;
                            Shift: TShiftState;
                             X, Y: Integer);
var
   ACol, ARow: longint;
begin
   inherited;
   MouseToCell(X, Y, ACol, ARow);

   // mouse up in same header as started then "clicked"
   if FColumnClick = ACol then begin
      // adjust column index to same as in grid.columns[] array
      if dgIndicator in Options then
         dec(FColumnClick);

      if (FColumnClick >= 0) and
         Assigned(FOnColumnClick)
      then FOnColumnClick(self, FColumnClick, FButton, FShift);

      FColumnClick := -1;  // finish tracking "click"
   end;
end;

{ WIN message handlers }

procedure TADBGrid.WMLButtonDown(var Message: TWMLButtonDown);
begin
   inherited;
   DoMouseDown(Message, mbLeft, []);
end;

procedure TADBGrid.WMRButtonDown(var Message: TWMRButtonDown);
begin
   inherited;
   DoMouseDown(Message, mbRight, []);
end;

procedure TADBGrid.WMMButtonDown(var Message: TWMMButtonDown);
begin
   inherited;
   DoMouseDown(Message, mbMiddle, []);
end;


{ public methods }

procedure TADBGrid.MouseToCell(X, Y: Integer; var ACol, ARow: Longint);
// copied frm TStringGrid uses MouseCoord protected method in TCustomGrid
var
  Coord: TGridCoord;
begin
  Coord := MouseCoord(X, Y);
  ACol := Coord.X;
  ARow := Coord.Y;
end;



procedure Register;
begin
  RegisterComponents('Samples', [TADBGrid]);
end;

end.
